/*
    Video maid
    copyright (c) 1998-2004 Kazuki IWAMOTO http://www.maid.org/ iwm@maid.org

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
#ifndef __VMAID_H__
#define __VMAID_H__


#include "common.h"
#include "avicore/avicore.h"
#ifdef G_OS_WIN32
# include <tchar.h>
# include <windows.h>
#endif /* G_OS_WIN32 */


#ifdef __cplusplus
extern "C" {
#endif /* __cplusplus */


#define VMAID_VERSION_NOTICE "Video maid "VERSION" ("VMAID_BUILD_ENVIRONMENT")"
#define VMAID_COPYRIGHT_NOTICE "Copyright (C) 1998-2004 Kazuki IWAMOTO"

#define MENUFILE 9


typedef struct VmaidCursor_Tag
{
  gint stream;                  /* ja:ストリームの位置(0,1) */
  gint frame;                   /* ja:現在の位置(0...) */
} VmaidCursor;
typedef struct VmaidHistory_Tag
{
  AviEdit *avi_edit[2];         /* ja:AVI編集 */
  VmaidCursor cursor, select;
  struct VmaidHistory_Tag *next;
} VmaidHistory;
typedef struct VmaidWindow_Tag
{
  gboolean edit;                /* ja:TRUE:編集済み,FALSE:未編集 */
  gchar *file;                  /* ja:ファイル名 */
  gint same;                    /* ja:-1:重複なし,0?:番号 */
  gint top;                     /* ja:ウインドウの先頭 */
  gint width, height;           /* ja:フレームのサイズ */
  guint rate, scale;            /* ja:レート÷スケール=1秒間のフレーム数 */
  guint timer_id;               /* ja:スクロール用タイマ */
  AviEdit *avi_edit[2];         /* ja:AVI編集 */
  VmaidCursor cursor;           /* ja:キャレットの位置 */
  VmaidCursor select;           /* ja:選択開始位置,-1:選択なし */
  VmaidHistory *undo, *redo;    /* ja:編集履歴 */
  GtkWidget *drawing;           /* ja:描画領域 */
  GtkWidget *hscroll;           /* ja:水平スクロールバー */
  GtkWidget *label;             /* ja:タブのラベル */
  GtkWidget *mark;              /* ja:タブの編集済みマーク */
  GtkWidget *menu_item;         /* ja:メニューのウインドウの項目 */
} VmaidWindow;


enum
{
  TARGET_VMAID
};


extern gboolean caret;                  /* ja:キャレットの形 */
extern gboolean newfile;                /* ja:TRUE:新規ファイル,FALSE:なし */
extern gboolean second;                 /* ja:TRUE:新規,FALSE:既存 */
extern gboolean default_ratio;          /* ja:TRUE:縦/横の比率を保持しない */
extern gboolean file_merge;             /* ja:TRUE:連番,FALSE:単独 */
extern gchar *history_files[10];        /* ja:ファイルの履歴 */
extern gchar *open_path;                /* ja:開くのパス */
extern gchar *save_path;                /* ja:名前を付けて保存のパス */
extern gchar *clipboard_scenario;       /* ja:クリップボードのテキスト */
extern gint def_width,def_height;       /* ja:デフォルトのウインドウのサイズ */
extern gint history;                    /* ja:ファイルの履歴 */
extern gint default_view;               /* ja:ウインドウが作成時の初期値 */
extern guint default_rate;              /* ja:ウインドウが作成時の初期値 */
extern guint timer_id;                  /* ja:タイマID */
extern GdkAtom atom_targets;            /* ja:ターゲット */
extern GdkAtom atom_scenario;           /* ja:シナリオオブジェクト */
extern GdkBitmap *bitmap0;              /* ja:波背景 */
extern GdkBitmap *bitmap1;              /* ja:網背景 */
extern GtkWidget *window;               /* ja:ウインドウ */
extern GtkWidget *notebook;             /* ja:ノートブック */
extern GtkWidget *tview;                /* ja:リストボックス */
extern GtkWidget *status;               /* ja:ステータスウインドウ */
extern GtkPositionType n_pos;           /* ja:ノートブックの位置 */
extern GtkItemFactory *ifactory_menu;   /* ja:メニュー */
extern GtkItemFactory *ifactory_popup;  /* ja:ポップアップ */
extern GtkItemFactory *ifactory_tview;  /* ja:ポップアップ */
extern GdkFont *system_font;            /* ja:フォント */
extern gint system_font_height;         /* ja:フォントの高さ */
extern gint system_font_ascent;         /* ja:フォントのアクセントの高さ */
extern GtkListStore *list_store;        /* ja:リスト */
extern GdkColor system_color[4];
#ifdef G_OS_WIN32
extern UINT uFormat;                    /* ja:クリップボード */
#endif /* G_OS_WIN32 */


#ifdef __cplusplus
}
#endif /* __cplusplus */


#endif
